@@ -2,16 +2,31 @@ from django.contrib import admin
 
 from quartet_epcis.models import entries, events, headers
 from django.utils.safestring import mark_safe
+from django.core.paginator import Paginator
+from django.conf import settings
+
+
+class NoCountPaginator(Paginator):
+    @property
+    def count(self):
+        return 100 * 10  # Some arbitrarily large number,
+        # so we can still get our page tab.
+
+
+use_no_count = getattr(settings, 'OPTIMIZE_HIGH_COUNT_MODEL_ADMINS', True)
+
 
 @admin.register(entries.Entry)
 class EntryAdmin(admin.ModelAdmin):
+    paginator = NoCountPaginator if use_no_count else Paginator
     list_display = (
         'identifier',
         'last_disposition',
         'parent_id',
         'top_id',
         'last_event',
-        'last_event_time'
+        'last_event_time',
+        'created'
     )
     raw_id_fields = (
         'top_id',
@@ -36,8 +51,12 @@ class EntryAdmin(admin.ModelAdmin):
 
 @admin.register(entries.EntryEvent)
 class EntryEventAdmin(admin.ModelAdmin):
+    paginator = NoCountPaginator if use_no_count else Paginator
+
     def url(self):
-        return mark_safe('<a class="download-task" href="%s%s">Download</a>' % ('/capture/task-data/', self.task_name))
+        return mark_safe(
+            '<a class="download-task" href="%s%s">Download</a>' % (
+                '/capture/task-data/', self.task_name))
 
     list_display = (
         'identifier',
@@ -49,6 +68,7 @@ class EntryEventAdmin(admin.ModelAdmin):
     search_fields = ['identifier']
     readonly_fields = (
         'event_type',
+        'event_time',
         'entry',
         'event',
         'is_parent',
@@ -108,6 +128,7 @@ class DestinationAdmin(admin.ModelAdmin):
 class DestinationEventAdmin(admin.ModelAdmin):
     pass
 
+
 @admin.register(headers.SBDH)
 class SBDHAdmin(admin.ModelAdmin):
     raw_id_fields = (
@@ -127,21 +148,25 @@ class PartnerAdmin(admin.ModelAdmin):
 class DocumentIdentificationAdmin(admin.ModelAdmin):
     pass
 
+
 class ILMDInline(admin.TabularInline):
     model = events.InstanceLotMasterData
     raw_id_fields = ('event',)
     extra = 0
 
+
 @admin.register(events.Event)
 class EventAdmin(admin.ModelAdmin):
+    paginator = NoCountPaginator if use_no_count else Paginator
     list_display = (
         'action', 'biz_step', 'disposition', 'read_point', 'biz_location',
-        'type'
+        'type', 'created'
     )
     inlines = [
         BusinessTransactionInline,
         ILMDInline
     ]
+    search_fields = ['created']
 
 
 def register_to_site(admin_site):
@@ -156,6 +181,7 @@ def register_to_site(admin_site):
                         InstanceLotMasterDataAdmin)
     admin_site.register(events.BusinessTransaction, BusinessTransactionAdmin)
     admin_site.register(events.QuantityElement, QuantityElementAdmin)
-    admin_site.register(headers.DocumentIdentification, DocumentIdentificationAdmin)
+    admin_site.register(headers.DocumentIdentification,
+                        DocumentIdentificationAdmin)
     admin_site.register(headers.Partner, PartnerAdmin)
     admin_site.register(headers.SBDH, SBDHAdmin)
